const getGQL = url =>
(query, variables) => fetch(url, {
    method: 'POST',
    headers: {
        "Content-Type": "application/json",
        ...(localStorage.authToken ? {"Authorization": "Bearer " + localStorage.authToken} : {})
    },
    body: JSON.stringify({query, variables})
}).then(res => res.json())
    .then(data => {
        if (data.data){
            return Object.values(data.data)[0] 
        } 
        else throw new Error(JSON.stringify(data.errors))
    })

export const backendURL = 'http://hipstagram.asmer.fs.a-level.com.ua';

const gql = getGQL(backendURL + '/graphql');





export const actionPromise = (name,promise) =>{
 const actionPending = name => ({name,type:"PROMISE",status:'PENDING'})
 const actionFulfiled = (name,payload) =>({name,type:"PROMISE",status:'FULFILLED',payload})
 const actionRejected = (name,error) => ({name,type:"PROMISE",status:'REJECTED',error})


 return async dispatch => {
    dispatch(actionPending(name))
    try{
        let payload = await promise
        dispatch(actionFulfiled(name,payload))
        return payload
    }
    catch(e){
        dispatch(actionRejected(name,e))
    }
 }
}


export const actionAuthLogin = (tokennn) =>({type:'AUTH_LOGIN',token:tokennn})
export const actionAuthLogout= () => ({type: 'AUTH_LOGOUT'})







export const actionFullRegister = (login,password) => async (dispatch, getState) => {
      await dispatch(
        actionPromise('fullRegister', gql(`mutation register($login:String!,$password:String!){
          UserUpsert(user:{login:$login,password:$password}){
            _id login
          }
        }`,{login: login, password: password}))
      );
      const {
        promise: {fullRegister}
      } = getState();
      if (fullRegister.status === "FULLFILLED") {
          dispatch(actionFullLogin(login,password))
      }
}
          

export const actionFullLogin = (login,password) => async (dispatch,getState) => {
   const tokennn = await dispatch(
    actionPromise('fullLogin', gql(`query login($login:String!,$password:String!){
        login(login:$login,password:$password)
    }`,{login:login,password:password}))
   );
   await dispatch(actionAuthLogin(tokennn));
   const {
     auth: {payload}
   } = getState();
   await dispatch(actionAboutMe(payload.sub.id));
}


export const uploadFile = file => {
  const fd = new FormData();
  fd.append('photo', file);
return  fetch(backendURL+'/upload', {
    method: "POST",
    headers: localStorage.authToken ? {Authorization: 'Bearer ' + localStorage.authToken} : {},
    body: fd
}).then(res => res.json());

}



export const actionUploadFile = file => 
  actionPromise('fileUpload', uploadFile(file));

export const actionUploadFiles = files =>
   actionPromise('filesUpload',Promise.all(files.map(file => uploadFile(file))))

export const actionAvatar = (userrId, fileeId) => 
  actionPromise("userAvatar", gql(`mutation setAvatar($userId: String, $fileId: ID){
    UserUpsert(user:{_id: $userId, avatar: {_id: $fileId}}){
        _id, avatar{
            _id
        }
    }
}`, {userId: userrId,
      fileId: fileeId}))

export const actionAboutMe = (_id) => 
        actionPromise("aboutMe", gql(`query AboutMe ($Id: String){
            UserFindOne(query:$Id){
              _id,nick, avatar{
                url
              },
              following{
                _id
              }
            }
          }`,{
              Id: JSON.stringify([{_id}])
          }))

export const actionAboutUser = (_id) =>
        actionPromise("aboutUser", gql(`query AboutUser ($Id: String){
          UserFindOne(query:$Id){
            _id,nick, avatar{
              url
            },
            followers {
              _id nick
            }
            following{
              _id nick
            }
          }
        }`,{
            Id: JSON.stringify([{_id}])
        }))          

export const actionSetAvatar = file => 
          async(dispatch, getState) => {
             await dispatch(actionUploadFile(file))
             const {auth:{payload:{sub:{id}}},promise:{fileUpload:{payload:{_id}}}} = getState() 
            await dispatch(actionAvatar(id,_id))
            await dispatch(actionAboutMe(id))
          }
export const actionPostUpsert = post =>
          actionPromise("postUpsert", gql(`mutation PostUpsert ($post: PostInput){
            PostUpsert(post:$post){
              _id
            }
          }`, {post:{...post,images:post.images.map(({_id})=>({_id}))}}))



export const actionFindUser = nick =>
          actionPromise("findUsers", gql(`query FindUsers($query: String){
            UserFind(query:$query){
              _id,nick,avatar{
                url
              }
            }
          }`, {query: JSON.stringify([
            {
              $or: [{nick:`/${nick}/`}]
            },
            {
              sort:[{nick:1}]
            }
          ])}))
          
export const actionUserPost = _id =>
        actionPromise("userPost", gql(`query PostFind($filter: String){
          PostFind(query:$filter){
                    owner{_id nick avatar{url}}
                    _id likesCount title images{_id url}
                      comments{
                        _id text post{_id} likesCount owner{_id nick avatar{url}}
                        
                      }
                    likes{
                        owner{				
                            login avatar {url}
                          }
                    }
              }
            }`,{
              filter: JSON.stringify([
                {
                  ___owner:{$in:[_id]}
                }
              ])
            }))      
            
export const actionFindOnePost = _id => 
            actionPromise("findPost", gql(`query FindOnePost($postId: String) {
              PostFindOne(query:$postId){
                  _id, title,text, images {
                   url, _id
                  }, comments{
                  _id text owner{_id nick} likesCount likes{
                    owner{
                      nick avatar{url}
                    }
                  }, answers{
                    _id, text, owner{
                      nick avatar{url}
                    }
                  }
                }, likes {_id owner{_id,nick}}, likesCount,
                owner {
                  _id, nick avatar{url}
                }
              }
            }`,{
              postId: JSON.stringify([{_id}])
            } ))

export const actionAddlike = _id => 
            actionPromise("addLike", gql(`mutation AddLike($like:LikeInput){
              LikeUpsert(like:$like){
                _id
              }
            }`,{
              like:{
                  "post":{
                      "_id": _id
                  }
              }
            }))

export const actionSubscribe = (userId,followId,oldFollow) =>
            actionPromise('subscribe', gql(`mutation Subscribe($user:UserInput){
              UserUpsert(user:$user){
                  following{_id, nick}
              }
            }`,
            {
              user: {
                  "_id": userId,
                  "following": [...(oldFollow || []), {"_id":followId}]
              }
            }))
export const actionFullSubscribe = (userId,followId) => async(dispatch,getState) => {
  const oldFollowing = (getState().promise.aboutMe?.payload?.following || []).map(item => ({
    _id: item._id
  }))
  let followingId = await dispatch(actionSubscribe(userId,followId, oldFollowing));
  if(followingId){
    await dispatch(actionAboutUser(followId))
    await dispatch(actionAboutMe(userId))
  }

}

export const actionUnSubscribe = (userId,oldFollow) =>
      actionPromise('unSubscribe', gql(`mutation UnSubscribe($user:UserInput){
        UserUpsert(user:$user){
          following{_id}
        }
      }`, {
        user: {
          "_id": userId,
          "following":[...oldFollow]
        }
      }))
export const actionFullUnSubscribe = (userId, followId) => async(dispatch,getState)=> {
  const oldFollowing = (getState().promise.aboutMe?.payload?.following || []).filter(item => item._id !==followId)
  const oldFollow = (oldFollowing || []).map(item => ({
    _id: item._id
  }))
  if(oldFollow){
    await dispatch(actionUnSubscribe(userId, oldFollow))
    await dispatch(actionAboutUser(followId));
    await dispatch(actionAboutMe(userId));
  }
}

export const actionDeleteLike = (_id) =>
        actionPromise('unLike', gql(`mutation Deletelike($like:LikeInput){
          LikeDelete(like:$like){
            _id post{_id owner{_id}}
          }
        }`,{
            like:{
              _id
            }
        }))

export const actionfullDeleteLike = (likeId) => async(dispatch,getState) => {
    let unLike = await dispatch(actionDeleteLike(likeId));
    if(unLike){
      await dispatch(actionFindOnePost(unLike?.post?._id));
    }
}        

export const actionAddFullLike = (postId) => async(dispatch,getState) => {
      let liked = await dispatch(actionAddlike(postId));
      if(liked){
        await dispatch(actionFindOnePost(postId));
      }
}

export const actionAddComment = (text,postId) => 
    actionPromise("addComment", gql(`mutation AddComment($comment: CommentInput){
      CommentUpsert(comment:$comment){
        _id 
      }
    }`,{
      comment:{
          "text":text,
          "post":{"_id":postId}
      }
    }))


export const actionFullComment = (text,postId) => async(dispatch,getState) => {
  let commented = await dispatch(actionAddComment(text,postId));
  if(commented){
    await dispatch(actionFindOnePost(postId));
  }
}

export const actionAddSubComment = (text,commentId,answers) => 
      actionPromise("addSubComment", gql(`mutation AddSubComment($comment: CommentInput){
        CommentUpsert(comment:$comment){
          _id
        }
      }`,{ comment:{
          "_id":commentId,
          "answers":[...(answers||[]),{"text":text}]
      }
      }))

export const actionfullAddSubComment = (text,commentId) => async(dispatch,getState)=> {
    const oldAnswers = (getState().promise?.findPost?.comments?.answers || []).map(item => ({_id:item._id}));
    console.log(oldAnswers);
    const postId = (getState().promise?.findPost?.payload?._id || '') 
    let answered = await dispatch(actionAddSubComment(text,commentId,oldAnswers));
    if(answered){
      await dispatch(actionFindOnePost(postId));
    }

}